/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.geoviewer.jxmap.layerlist;

import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.LayerInfo;
import nl.cloudfarming.client.geoviewer.LayerNode;
import org.openide.nodes.Node;

/**
 * layer controller, controls the layerlist
 * 
 * @author Timon Veenstra
 */
public interface LayerController {
    
    /**
     * add a layernode to the layerlist
     * 
     * @param layerNode 
     */
    void addLayerNode(Node layerNode);
    
    /**
     * remove a layernode from the layerlist
     * 
     * @param layerNode 
     */
    void removeLayerNode(Node layerNode);
    
    /**
     * change layer display information
     * 
     * @param layerInfo 
     */
    void changeLayerInfo(LayerInfo layerInfo);
    
    /**
     * get all managed layernodes
     * 
     * @return 
     */
    Node[] getLayerNodes();

    /**
     * get display information for a layer
     * 
     * @param layer
     * @return 
     */
    public LayerInfo getLayerInfo(Layer layer);
}
