/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.geoviewer.jxmap.map;

import java.beans.PropertyChangeEvent;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.MapController;
import nl.cloudfarming.client.geoviewer.ObjectLayer;
import org.jdesktop.swingx.JXMapViewer;
import org.openide.nodes.Node;
import org.openide.nodes.NodeEvent;
import org.openide.nodes.NodeListener;
import org.openide.nodes.NodeMemberEvent;
import org.openide.nodes.NodeReorderEvent;

/**
 *
 * @author Timon Veenstra
 */
public class MapPanelFactory {

    private final JXMapViewer mapViewer;
    private final MapController mapController;
    private static int id;

    public MapPanelFactory(JXMapViewer mapViewer, MapController mapController) {
        this.mapViewer = mapViewer;
        this.mapController = mapController;
    }

    public LayerPanel createPanel(final Node layerNode) {
        LayerPanel layerPanel;
        final Layer layer = layerNode.getLookup().lookup(Layer.class);
        if (layer instanceof ObjectLayer) {
            layerPanel = new ObjectLayerPanel(mapController,  layerNode,layer, mapViewer, id++);
            layerNode.addNodeListener(new NodeListener() {

                @Override
                public void childrenAdded(NodeMemberEvent ev) {
                    mapController.refreshLayer(layer);
                }

                @Override
                public void childrenRemoved(NodeMemberEvent ev) {
                    mapController.refreshLayer(layer);
                }

                @Override
                public void childrenReordered(NodeReorderEvent ev) {
                }

                @Override
                public void nodeDestroyed(NodeEvent ev) {
                    mapController.refreshLayer(layer);
                }

                @Override
                public void propertyChange(PropertyChangeEvent evt) {
                }
            });


        } else {
            layerPanel = new ImageLayerPanel(mapController, layer, mapViewer, id++);
        }
        return layerPanel;
    }
}
