/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package nl.cloudfarming.client.geoviewer.jxmap.render;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import org.jdesktop.swingx.JXMapViewer;
import org.jdesktop.swingx.mapviewer.GeoPosition;

/**
 *
 * @author Timon Veenstra
 */
//TODO refactor to extend org.jdesktop.swingx.painter.Painter 
public abstract class GeometryRenderer<G extends Geometry> {

    /**
     * geometry renderer, paint the geometry
     *
     * @param geometry
     * @param mapViewer
     * @param graphics
     * @param selected
     * @return rectangle (nearest boundingbox around the geometry)
     */
    public abstract Rectangle paint(G geometry, JXMapViewer mapViewer, Graphics2D graphics, boolean selected);
    
    /**
     * Calculate viewport from specified mapviewer and geometry
     * @param geometry The geometry to calculate a viewport for
     * @param mapViewer The mapviewer to get viewportbounds from
     * @return 
     */
    protected Rectangle getViewport(G geometry, JXMapViewer mapViewer){       
        //
        // determine the box the geometry will be drawn in
        //
        Rectangle bounds = new Rectangle(0, 0, -1, -1);
        boolean firstpoint = false;

        for (Coordinate boundryCoord : geometry.getBoundary().getCoordinates()) {
            GeoPosition geopoint = new GeoPosition(boundryCoord.y, boundryCoord.x);
            Point2D point2D = mapViewer.getTileFactory().geoToPixel(geopoint, mapViewer.getZoom());

            int x = (int) (point2D.getX() - mapViewer.getViewportBounds().getX());
            int y = (int) (point2D.getY() - mapViewer.getViewportBounds().getY());
            if (firstpoint) {
                bounds.setLocation(x, y);
            }
            bounds.add(x, y);
        }
        return bounds;
    }
}
